# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to update labels for external VPN gateway."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute.external_vpn_gateways import external_vpn_gateways_utils
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions as calliope_exceptions
from googlecloudsdk.command_lib.compute import flags as compute_flags
from googlecloudsdk.command_lib.compute.external_vpn_gateways import flags as external_vpn_gateway_flag
from googlecloudsdk.command_lib.util.args import labels_util


_EXTERNAL_VPN_GATEWAY_ARG = (
    external_vpn_gateway_flag.ExternalVpnGatewayArgument())


@base.ReleaseTracks(
    base.ReleaseTrack.ALPHA,
    base.ReleaseTrack.BETA,
    base.ReleaseTrack.GA,
    base.ReleaseTrack.PREVIEW,
)
class Update(base.UpdateCommand):
  r"""Update a Compute Engine external VPN gateway.

  *{command}* updates labels for a Compute Engine external VPN gateway.
  For example:

    $ {command} example-gateway \
      --update-labels=k0=value1,k1=value2 --remove-labels=k3

  will add/update labels ``k0'' and ``k1'' and remove labels with key ``k3''.

  Labels can be used to identify the External VPN gateway and to filter them
  as in

    $ {parent_command} list --filter='labels.k1:value2'

  To list existing labels

    $ {parent_command} describe example-gateway --format="default(labels)"

  """

  detailed_help = {
      'EXAMPLES':
          """\
          To update labels for an external VPN gateway, run:

              $ {command} my-external-gateway \
                --update-labels=k0=value1,k1=value2"""
  }

  @classmethod
  def Args(cls, parser):
    """Adds arguments to the supplied parser.

    Args:
      parser: The argparse parser to add arguments to.
    """
    _EXTERNAL_VPN_GATEWAY_ARG.AddArgument(parser, operation_type='update')
    labels_util.AddUpdateLabelsFlags(parser)

  def Run(self, args):
    """Issues API requests to update a External VPN gateway.

    Args:
      args: argparse.Namespace, The arguments received by this command.
    Returns:
      [protorpc.messages.Message], A list of responses returned
      by the compute API.
    """
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    messages = holder.client.messages
    helper = external_vpn_gateways_utils.ExternalVpnGatewayHelper(holder)
    external_gateway_ref = _EXTERNAL_VPN_GATEWAY_ARG.ResolveAsResource(
        args,
        holder.resources,
        scope_lister=compute_flags.GetDefaultScopeLister(holder.client))

    labels_diff = labels_util.Diff.FromUpdateArgs(args)
    if not labels_diff.MayHaveUpdates():
      raise calliope_exceptions.RequiredArgumentException(
          'LABELS', 'At least one of --update-labels or '
                    '--remove-labels must be specified.')

    external_vpn_gateway = helper.Describe(external_gateway_ref)
    labels_update = labels_diff.Apply(
        messages.GlobalSetLabelsRequest.LabelsValue,
        external_vpn_gateway.labels)

    if not labels_update.needs_update:
      return external_vpn_gateway

    operation_ref = helper.SetLabels(external_gateway_ref,
                                     external_vpn_gateway.labelFingerprint,
                                     labels_update.labels)
    return helper.WaitForOperation(
        external_gateway_ref, operation_ref,
        'Updating labels of external VPN gateway [{0}]'.format(
            external_gateway_ref.Name()))



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Shared resource arguments and flags."""


from googlecloudsdk.calliope import arg_parsers


def AddFullUri(parser, positional=False, help_text=None, **kwargs):
  """Add full uri argument to the parser."""

  parser.add_argument(
      _ArgOrFlag('full-uri', positional), help=(help_text), **kwargs
  )


def AddFloorSettingEnforcement(parser, positional=False, **kwargs):
  """Enable or disable the floor setting enforcement."""

  parser.add_argument(
      _ArgOrFlag('enable-floor-setting-enforcement', positional),
      help=(
          'Enable or disable the floor setting enforcement. Set the value to'
          ' "TRUE" to enable the floor setting enforcement, "FALSE" to'
          ' disable it.'
      ),
      **kwargs
  )


def AddMaliciousUriFilterSettingsEnforcement(parser):
  """Add malicious uri filter settings enforcement argument to the parser."""
  group = parser.add_group(mutex=True, help='Malicious uri filter settings.')
  group.add_argument(
      _ArgOrFlag('malicious-uri-filter-settings-enforcement', False),
      metavar='MALICIOUS_URI_FILTER_SETTINGS_ENFORCEMENT',
      help='Malicious URI filter settings.',
  )


def AddPIJBFilterSettingsGroup(parser):
  """Add flags for specifying pi and jailbreak filter settings."""
  group = parser.add_group(
      mutex=False, help='PI and jailbreak filter settings.'
  )
  group.add_argument(
      _ArgOrFlag('pi-and-jailbreak-filter-settings-enforcement', False),
      metavar='PI_AND_JAILBREAK_FILTER_SETTINGS_ENFORCEMENT',
      help=(
          'The pi and jailbreak filter settings enforcement. The value can be'
          ' either "enable" or "disable".'
      ),
  )
  group.add_argument(
      _ArgOrFlag('pi-and-jailbreak-filter-settings-confidence-level', False),
      metavar='PI_AND_JAILBREAK_FILTER_SETTINGS_CONFIDENCE_LEVEL',
      help=(
          'The pi and jailbreak filter settings confidence level. The value can'
          ' be either "high", "medium-and-above" or "low-and-above"'
      ),
  )


def AddSDPFilterBasicConfigGroup(parser):
  """Add flags for specifying sdp filter settings."""
  group = parser.add_group(mutex=False, help='SDP filter settings.')
  group.add_argument(
      _ArgOrFlag('basic-config-filter-enforcement', False),
      metavar='BASIC_CONFIG_FILTER_ENFORCEMENT',
      help=(
          'The sdp filter settings enforcement. The value can be either'
          ' `ENABLED` or `DISABLED`.'
      ),
  )
  group.add_argument(
      _ArgOrFlag('advanced-config-inspect-template', False),
      metavar='ADVANCED_CONFIG_INSPECT_TEMPLATE',
      help=(
          'The sdp filter settings enforcement. The value has to be a valid'
          ' template name. e.g.'
          ' `projects/{project}/locations/{location}/inspectTemplates/{inspect_template}`'
      ),
  )
  group.add_argument(
      _ArgOrFlag('advanced-config-deidentify-template', False),
      metavar='ADVANCED_CONFIG_DEIDENTIFY_TEMPLATE',
      help=(
          'The sdp filter settings enforcement. The value has to be a valid'
          ' template name. e.g.'
          ' `projects/{project}/locations/{location}/deidentifyTemplates/{deidentify_template}`'
      ),
  )


def AddRaiFilterSettingsGroup(parser):
  """Add flags for specifying rai filter settings."""
  group = parser.add_group(mutex=True, help='RAI filter settings.')
  group.add_argument(
      _ArgOrFlag('rai-settings-filters', False),
      metavar='confidenceLevel=CONFIDENCELEVEL],[filterType=FILTERTYPE]',
      type=arg_parsers.ArgObject(repeated=True),
      action=arg_parsers.FlattenAction(),
      help=(
          'Set rai_settings_filters to new value. List of Responsible AI'
          ' filters enabled for floor setting'
      ),
  )
  group.add_argument(
      _ArgOrFlag('add-rai-settings-filters', False),
      metavar='confidenceLevel=CONFIDENCELEVEL],[filterType=FILTERTYPE]',
      type=arg_parsers.ArgObject(repeated=True),
      action=arg_parsers.FlattenAction(),
      help='Add rai filter settings.',
  )
  group.add_argument(
      _ArgOrFlag('remove-rai-settings-filters', False),
      metavar='confidenceLevel=CONFIDENCELEVEL],[filterType=FILTERTYPE]',
      type=arg_parsers.ArgObject(repeated=True),
      action=arg_parsers.FlattenAction(),
      help='Remove rai filter settings.',
  )
  group.add_argument(
      _ArgOrFlag('clear-rai-settings-filters', False),
      action='store_true',
      help='Clear all rai filter settings.',
  )


def AddIntegratedServices(parser):
  """Add flags for specifying integrated services."""
  group = parser.add_group(
      mutex=True,
      help=(
          'Manage integrated services to apply Model Armor floor settings.'
          ' Integrated services will have Model Armor sanitization enabled'
          ' project-wide.'
      ),
  )
  group.add_argument(
      _ArgOrFlag('add-integrated-services', False),
      metavar='INTEGRATED_SERVICE',
      type=arg_parsers.ArgList(),
      action=arg_parsers.FlattenAction(),
      help=(
          'Set the list of integrated services for the floor setting. This can'
          ' be used to enable project-wide Model Armor sanitization for the'
          ' respective services. This flag can be repeated to specify'
          ' multiple services, or a comma-separated list can be provided.'
      ),
  )
  group.add_argument(
      _ArgOrFlag('remove-integrated-services', False),
      metavar='INTEGRATED_SERVICE',
      type=arg_parsers.ArgList(),
      action=arg_parsers.FlattenAction(),
      help='Remove specified service(s) from the list of integrated services.'
           ' This flag can be repeated to specify multiple services, or a'
           ' comma-separated list can be provided.',
  )
  group.add_argument(
      _ArgOrFlag('clear-integrated-services', False),
      action='store_true',
      help='Clear all integrated services from the floor setting.',
  )


def AddVertexAiFloorSetting(parser):
  """Add flags for specifying vertex ai floor setting."""
  group = parser.add_group(
      mutex=False, help='Options for Vertex AI sanitization.'
  )
  group.add_argument(
      '--enable-vertex-ai-cloud-logging',
      dest='enable_vertex_ai_cloud_logging',
      help=(
          'Enable Cloud Logging for Vertex AI sanitization to log Model'
          ' Armor sanitization results.'
      ),
      action=arg_parsers.StoreTrueFalseAction,
  )
  group.add_argument(
      _ArgOrFlag('vertex-ai-enforcement-type', False),
      dest='vertex_ai_enforcement_type',
      help=(
          'Specifies the enforcement mode for Vertex AI sanitization, such as'
          ' "INSPECT_ONLY" or "INSPECT_AND_BLOCK".'
          ' Default is "INSPECT_ONLY".'
      ),
  )


def AddGoogleMcpServerFloorSetting(parser) -> None:
  """Add flags for specifying Google MCP server floor setting."""
  group = parser.add_group(
      mutex=False, help='Options for Google MCP server sanitization.'
  )
  group.add_argument(
      '--enable-google-mcp-server-cloud-logging',
      dest='enable_google_mcp_server_cloud_logging',
      help=(
          'Enable Cloud Logging for Google MCP server sanitization to log'
          ' Model Armor sanitization results.'
      ),
      action=arg_parsers.StoreTrueFalseAction,
  )
  group.add_argument(
      _ArgOrFlag('google-mcp-server-enforcement-type', False),
      dest='google_mcp_server_enforcement_type',
      help=(
          'Specifies the enforcement mode for Google MCP server sanitization,'
          ' such as "INSPECT_ONLY" or "INSPECT_AND_BLOCK". Default is'
          ' "INSPECT_ONLY".'
      ),
  )


def AddGoogleMcpServerApiFlags(parser):
  """Add flags for Google MCP server APIs."""
  apis_group = parser.add_group(
      mutex=True,
      help=(
          'Manage list of MCP servers for which the Google MCP server floor'
          ' setting is applicable. Empty list indicates that the floor setting'
          ' is applicable to all Google MCP servers. APIs need to be valid MCP'
          ' resources, for example:'
          '`"bigquery.googleapis.com/mcp"`, `"run.googleapis.com/mcp"`'
      ),
  )
  apis_group.add_argument(
      _ArgOrFlag('google-mcp-server-apis', False),
      metavar='API',
      type=arg_parsers.ArgList(),
      action=arg_parsers.FlattenAction(),
      help=(
          'Replaces the current list of Google MCP server APIs with the'
          ' provided list.'
      ),
  )
  apis_group.add_argument(
      _ArgOrFlag('add-google-mcp-server-apis', False),
      metavar='API',
      type=arg_parsers.ArgList(),
      action=arg_parsers.FlattenAction(),
      help='Appends the given APIs to the list of Google MCP server APIs.',
  )
  apis_group.add_argument(
      _ArgOrFlag('remove-google-mcp-server-apis', False),
      metavar='API',
      type=arg_parsers.ArgList(),
      action=arg_parsers.FlattenAction(),
      help='Removes the given APIs from the list of Google MCP server APIs.',
  )
  apis_group.add_argument(
      _ArgOrFlag('clear-google-mcp-server-apis', False),
      action='store_true',
      help=(
          'Removes all APIs from the list of Google MCP server APIs. Empty'
          ' list indicates that the floor setting is applicable to all Google'
          ' MCP servers.'
      ),
  )


def AddMultiLanguageDetection(parser):
  """Add flags for specifying multi language detection."""
  group = parser.add_group(
      mutex=False, help='Multi language detection enablement.'
  )
  group.add_argument(
      '--enable-multi-language-detection',
      dest='enable_multi_language_detection',
      help=(
          'Enable multi-language detection for floor setting, allowing Model'
          ' Armor to process content in multiple languages.'
      ),
      action=arg_parsers.StoreTrueFalseAction,
  )


def _ArgOrFlag(name, positional):
  """Returns the argument name in resource argument format or flag format.

  Args:
      name (str): name of the argument
      positional (bool): whether the argument is positional

  Returns:
      arg (str): the argument or flag
  """
  if positional:
    return name.upper().replace('-', '_')
  return '--{}'.format(name)



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Flags and helpers for the Cloud NetApp Backup Vaults commands."""


import textwrap

from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.netapp import flags
from googlecloudsdk.command_lib.util.apis import arg_utils
from googlecloudsdk.command_lib.util.args import labels_util
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.command_lib.util.concepts import presentation_specs


BACKUP_VAULTS_LIST_FORMAT = """\
    table(
        name.basename():label=BACKUP_VAULT_NAME:sort=1,
        name.segment(3):label=LOCATION,
        state
    )"""


def GetBackupVaultTypeEnumFromArg(choice, messages):
  """Returns the Choice Enum for Backup Vault Type.

  Args:
    choice: The choice for backup vault type as string
    messages: The messages module.

  Returns:
    the backup vault type enum.
  """
  return arg_utils.ChoiceToEnum(
      choice=choice,
      enum_type=messages.BackupVault.BackupVaultTypeValueValuesEnum,
  )


def AddBackupVaultTypeArg(parser):
  help_text = """\
  String indicating the type of backup vault.
  The supported values are: 'IN_REGION','CROSS_REGION'
  """

  parser.add_argument(
      '--backup-vault-type',
      type=str,
      help=help_text,
      required=False,
  )


def AddBackupVaultBackupLocationArg(parser):
  """Adds the Backup Vault Backup Location arg to the arg parser."""
  parser.add_argument(
      '--backup-region',
      type=str,
      help="""String indicating backup location for the Backup Vault""",
  )
## Helper functions to combine Backup Vaults args / flags for gcloud commands ##


def AddBackupVaultCreateArgs(parser, release_track):
  """Add args for creating a Backup Vault."""
  concept_parsers.ConceptParser(
      [flags.GetBackupVaultPresentationSpec('The Backup Vault to create')]
  ).AddToParser(parser)
  flags.AddResourceDescriptionArg(parser, 'Backup Vault')
  flags.AddResourceAsyncFlag(parser)
  labels_util.AddCreateLabelsFlags(parser)
  AddBackupRetentionPolicyArg(parser)
  AddKmsConfigArg(parser)
  if (release_track == base.ReleaseTrack.BETA):
    AddBackupVaultTypeArg(parser)
    AddBackupVaultBackupLocationArg(parser)


def AddBackupVaultDeleteArgs(parser):
  """Add args for deleting a Backup Vault."""
  concept_parsers.ConceptParser(
      [flags.GetBackupVaultPresentationSpec('The Backup Vault to delete')]
  ).AddToParser(parser)
  flags.AddResourceAsyncFlag(parser)


def AddBackupVaultUpdateArgs(parser):
  """Add args for updating a Backup Vault."""
  concept_parsers.ConceptParser(
      [flags.GetBackupVaultPresentationSpec('The Backup Vault to update')]
  ).AddToParser(parser)
  flags.AddResourceDescriptionArg(parser, 'Backup Vault')
  flags.AddResourceAsyncFlag(parser)
  labels_util.AddUpdateLabelsFlags(parser)
  AddBackupRetentionPolicyArg(parser)


def AddBackupRetentionPolicyArg(parser):
  """Adds the Backup Retention Policy arg to the arg parser."""
  backup_retention_policy_arg_spec = {
      'backup-minimum-enforced-retention-days': int,
      'daily-backup-immutable': bool,
      'weekly-backup-immutable': bool,
      'monthly-backup-immutable': bool,
      'manual-backup-immutable': bool
  }
  backup_retention_policy_help = textwrap.dedent("""\
    Backup Retention Policy of the Backup Vault.

    Backup Retention Policy allows you to configure the retention policy for
    backups created within this vault. It consists of several fields that govern
    how long backups are kept and what type of backups are immutable.
    """)
  parser.add_argument(
      '--backup-retention-policy',
      type=arg_parsers.ArgDict(spec=backup_retention_policy_arg_spec),
      required=False,
      help=backup_retention_policy_help,
      )


def AddKmsConfigArg(parser, hidden=False):
  """Adds the --kms-config flag to the parser."""
  help_text = """\
  The resource name of the KMS Config to use for encrypting backups within this backup vault.
  Format: projects/{project_id}/locations/{location}/kmsConfigs/{kms_config_id}
  """
  kms_config_presentation_spec = presentation_specs.ResourcePresentationSpec(
      '--kms-config',
      flags.GetKmsConfigResourceSpec(),
      help_text,
      required=False,
      hidden=hidden,
      flag_name_overrides={'location': ''},
  )
  concept_parsers.ConceptParser([kms_config_presentation_spec]).AddToParser(
      parser
  )



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Bare Metal Solution instance rename command."""


from googlecloudsdk.api_lib.bms.bms_client import BmsClient
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.bms import flags
from googlecloudsdk.core import log


DETAILED_HELP = {
    'DESCRIPTION':
        """
          Rename a Bare Metal Solution instance.
        """,
    'EXAMPLES':
        """
          To rename an instance ``my-instance'' to ``my-new-instance-name'' in region ``us-central1'', run:

          $ {command} my-instance --new-name=my-new-instance-name --region=us-central1 --project=bms-example-project
        """,
}


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.GA)
class Rename(base.UpdateCommand):
  """Rename a Bare Metal Solution instance."""

  @staticmethod
  def Args(parser):
    """Register flags for this command."""
    flags.AddInstanceArgToParser(parser, positional=True)
    flags.AddNewNameArgToParser(parser, 'instance')

  def Run(self, args):
    client = BmsClient()
    old_name = args.CONCEPTS.instance.Parse()
    new_name = args.new_name
    res = client.RenameInstance(old_name, new_name)
    log.status.Print(
        'Renamed {} to {} successfully'.format(old_name.Name(), new_name))
    return res


Rename.detailed_help = DETAILED_HELP



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Creates a Cloud NetApp Storage Pool."""


from googlecloudsdk.api_lib.netapp.storage_pools import client as storagepools_client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.netapp.storage_pools import flags as storagepools_flags
from googlecloudsdk.command_lib.util.args import labels_util

from googlecloudsdk.core import log


def _CommonArgs(parser, release_track):
  storagepools_flags.AddStoragePoolCreateArgs(
      parser, release_track=release_track
  )


@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.GA)
class Create(base.CreateCommand):
  """Create a Cloud NetApp Storage Pool."""

  _RELEASE_TRACK = base.ReleaseTrack.GA

  detailed_help = {
      'DESCRIPTION': """\
          Creates a Storage Pool to contain Volumes with a specified Service Level and capacity.
          """,
      'EXAMPLES': """\
          The following command creates a Storage Pool named NAME using all possible arguments with a VPC network in the same project

              $ {command} NAME --location=us-central1 --service-level=standard --capacity=2048 --network=name=default --active-directory=ad1 --kms-config=kms-config1 --enable-ldap=true --description="example description"

          The following command creates a Storage pool named NAME using all possible arguments with a shared VPC network in a separate project called VPC_PROJECT

              $ {command} NAME --location=us-central1 --service-level=standard --capacity=2048 --network=name=projects/VPC_PROJECT/locations/us-central1/networks/default --active-directory=ad1 --kms-config=kms-config1 --enable-ldap=true --description="example description"

          """,
  }

  @staticmethod
  def Args(parser):
    _CommonArgs(parser, Create._RELEASE_TRACK)

  def Run(self, args):
    """Create a Cloud NetApp Storage Pool in the current project."""
    storagepool_ref = args.CONCEPTS.storage_pool.Parse()
    client = storagepools_client.StoragePoolsClient(self._RELEASE_TRACK)
    service_level = storagepools_flags.GetStoragePoolServiceLevelArg(
        client.messages, self._RELEASE_TRACK
    ).GetEnumForChoice(args.service_level)
    labels = labels_util.ParseCreateArgs(
        args, client.messages.StoragePool.LabelsValue)
    capacity_in_gib = args.capacity >> 30
    zone = args.zone
    replica_zone = args.replica_zone
    if args.total_throughput is not None:
      total_throughput_mibps = args.total_throughput >> 20
    else:
      total_throughput_mibps = None
    hot_tier_size_gib = None
    qos_type = None
    if args.qos_type is not None:
      qos_type = storagepools_flags.GetStoragePoolQosTypeArg(
          client.messages
      ).GetEnumForChoice(args.qos_type)
    enable_hot_tier_auto_resize = None
    unified_pool = None
    storage_pool_type = None
    scale_tier = None
    scale_type = None
    mode = None
    if args.type is not None:
      storage_pool_type = storagepools_flags.GetStoragePoolTypeEnumFromArg(
          args.type, client.messages
      )
    if args.scale_type is not None:
      scale_type = storagepools_flags.GetStoragePoolScaleTypeEnumFromArg(
          args.scale_type, client.messages
      )
    if args.mode is not None:
      mode = storagepools_flags.GetStoragePoolModeEnumFromArg(
          args.mode, client.messages
      )
    if (self._RELEASE_TRACK == base.ReleaseTrack.ALPHA or
        self._RELEASE_TRACK == base.ReleaseTrack.BETA):
      if args.hot_tier_size is not None:
        hot_tier_size_gib = args.hot_tier_size >> 30
      enable_hot_tier_auto_resize = args.enable_hot_tier_auto_resize
      if args.unified_pool is not None:
        unified_pool = args.unified_pool
      if args.scale_tier is not None:
        scale_tier = storagepools_flags.GetStoragePoolScaleTierEnumFromArg(
            args.scale_tier, client.messages
        )

    storage_pool = client.ParseStoragePoolConfig(
        name=storagepool_ref.RelativeName(),
        service_level=service_level,
        network=args.network,
        active_directory=args.active_directory,
        kms_config=args.kms_config,
        enable_ldap=args.enable_ldap,
        capacity=capacity_in_gib,
        description=args.description,
        allow_auto_tiering=args.allow_auto_tiering,
        zone=zone,
        replica_zone=replica_zone,
        custom_performance_enabled=args.custom_performance_enabled,
        total_throughput=total_throughput_mibps,
        total_iops=args.total_iops,
        hot_tier_size=hot_tier_size_gib,
        enable_hot_tier_auto_resize=enable_hot_tier_auto_resize,
        labels=labels,
        unified_pool=unified_pool,
        qos_type=qos_type,
        storage_pool_type=storage_pool_type,
        scale_tier=scale_tier,
        scale_type=scale_type,
        mode=mode,
    )
    result = client.CreateStoragePool(
        storagepool_ref, args.async_, storage_pool
    )
    if args.async_:
      command = 'gcloud {} netapp storage-pools list'.format(
          self.ReleaseTrack().prefix
      )
      log.status.Print(
          'Check the status of the new storage pool by listing all storage'
          ' pools:\n  $ {} '.format(command)
      )
    return result


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class CreateBeta(Create):
  """Create a Cloud NetApp Storage Pool."""

  _RELEASE_TRACK = base.ReleaseTrack.BETA

  @staticmethod
  def Args(parser):
    _CommonArgs(parser, CreateBeta._RELEASE_TRACK)


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class CreateAlpha(CreateBeta):
  """Create a Cloud NetApp Storage Pool."""

  _RELEASE_TRACK = base.ReleaseTrack.ALPHA

  @staticmethod
  def Args(parser):
    _CommonArgs(parser, CreateAlpha._RELEASE_TRACK)



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Log utilities for storage insights dataset-configs commands."""


from googlecloudsdk.core import log


def dataset_config_operation_started_and_status_log(
    verb, dataset_config_name, operation_id
):
  # TODO: b/425284402 - change command to gcloud storage insights once GA.
  log.status.Print(
      '{} operation for dataset config {} has been successfully started.\nTo'
      ' check the status of this operation run: gcloud alpha storage insights'
      ' operations describe {}'.format(verb, dataset_config_name, operation_id)
  )



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Task for bulk restoring soft-deleted objects."""


from googlecloudsdk.api_lib.storage import api_factory
from googlecloudsdk.api_lib.storage import request_config_factory
from googlecloudsdk.command_lib.storage import progress_callbacks
from googlecloudsdk.command_lib.storage.tasks import task
from googlecloudsdk.core import log


class BulkRestoreObjectsTask(task.Task):
  """Restores soft-deleted cloud storage objects."""

  def __init__(
      self,
      bucket_url,
      object_globs,
      allow_overwrite=False,
      created_after_time=None,
      created_before_time=None,
      deleted_after_time=None,
      deleted_before_time=None,
      user_request_args=None,
  ):
    """Initializes task.

    Args:
      bucket_url (StorageUrl): Launch a bulk restore operation for this bucket.
      object_globs (list[str]): Objects in the target bucket matching these glob
        patterns will be restored.
      allow_overwrite (bool): Overwrite existing live objects.
      created_after_time (datetime): Filter results to objects created after
        this time.
      created_before_time (datetime): Filter results to objects created before
        this time.
      deleted_after_time (datetime): Filter results to objects soft-deleted
        after this time.
      deleted_before_time (datetime): Filter results to objects soft-deleted
        before this time.
      user_request_args (UserRequestArgs|None): Contains restore settings.
    """
    super(BulkRestoreObjectsTask, self).__init__()
    self._bucket_url = bucket_url
    self._object_globs = object_globs
    self._allow_overwrite = allow_overwrite
    self._created_after_time = created_after_time
    self._created_before_time = created_before_time
    self._deleted_after_time = deleted_after_time
    self._deleted_before_time = deleted_before_time
    self._user_request_args = user_request_args

  def execute(self, task_status_queue=None):
    log.status.Print(
        'Creating bulk restore operation for bucket {} with globs: {}'.format(
            self._bucket_url, self._object_globs
        )
    )
    request_config = request_config_factory.get_request_config(
        # Arbitrarily use first glob to get CloudUrl for object.
        self._bucket_url.join(self._object_globs[0]),
        user_request_args=self._user_request_args,
    )

    created_operation = api_factory.get_api(
        self._bucket_url.scheme
    ).bulk_restore_objects(
        self._bucket_url,
        self._object_globs,
        request_config=request_config,
        allow_overwrite=self._allow_overwrite,
        created_after_time=self._created_after_time,
        created_before_time=self._created_before_time,
        deleted_after_time=self._deleted_after_time,
        deleted_before_time=self._deleted_before_time,
    )

    log.status.Print('Created: ' + created_operation.name)
    if task_status_queue:
      progress_callbacks.increment_count_callback(task_status_queue)

  def __eq__(self, other):
    if not isinstance(other, type(self)):
      return NotImplemented
    return (
        self._bucket_url == other._bucket_url
        and self._object_globs == other._object_globs
        and self._allow_overwrite == other._allow_overwrite
        and self._created_after_time == other._created_after_time
        and self._created_before_time == other._created_before_time
        and self._deleted_after_time == other._deleted_after_time
        and self._deleted_before_time == other._deleted_before_time
        and self._user_request_args == other._user_request_args
    )



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""`gcloud api-gateway gateways create` command."""


from googlecloudsdk.api_lib.api_gateway import gateways
from googlecloudsdk.api_lib.api_gateway import operations
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.api_gateway import common_flags
from googlecloudsdk.command_lib.api_gateway import operations_util
from googlecloudsdk.command_lib.api_gateway import resource_args
from googlecloudsdk.command_lib.util.args import labels_util


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA,
                    base.ReleaseTrack.GA)
@base.DefaultUniverseOnly
class Create(base.CreateCommand):
  """Create a new gateway."""

  detailed_help = {
      'DESCRIPTION':
          '{description}',
      'EXAMPLES':
          """\
        To create a gateway in 'us-central1' run:

          $ {command} my-gateway --api=my-api --api-config=my-config --location=us-central1
        """,
  }

  @staticmethod
  def Args(parser):
    base.ASYNC_FLAG.AddToParser(parser)
    common_flags.AddDisplayNameArg(parser)
    labels_util.AddCreateLabelsFlags(parser)
    resource_args.AddGatewayApiConfigResourceArgs(parser, 'created')

  def Run(self, args):
    gateway_ref = args.CONCEPTS.gateway.Parse()
    api_config_ref = args.CONCEPTS.api_config.Parse()

    gateways_client = gateways.GatewayClient()
    resp = gateways_client.Create(gateway_ref,
                                  api_config_ref,
                                  display_name=args.display_name,
                                  labels=args.labels)

    wait = 'Waiting for API Gateway [{}] to be created with [{}] config'.format(
        gateway_ref.Name(), api_config_ref.RelativeName())

    return operations_util.PrintOperationResult(
        resp.name,
        operations.OperationsClient(),
        service=gateways_client.service,
        wait_string=wait,
        is_async=args.async_)



"""Generated message classes for policytroubleshooter version v1.

"""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding
from apitools.base.py import extra_types


package = 'policytroubleshooter'


class GoogleCloudPolicytroubleshooterV1AccessTuple(_messages.Message):
  r"""Information about the principal, resource, and permission to check.

  Fields:
    fullResourceName: Required. The full resource name that identifies the
      resource. For example, `//compute.googleapis.com/projects/my-
      project/zones/us-central1-a/instances/my-instance`. For examples of full
      resource names for Google Cloud services, see
      https://cloud.google.com/iam/help/troubleshooter/full-resource-names.
    permission: Required. The IAM permission to check for the specified
      principal and resource. For a complete list of IAM permissions, see
      https://cloud.google.com/iam/help/permissions/reference. For a complete
      list of predefined IAM roles and the permissions in each role, see
      https://cloud.google.com/iam/help/roles/reference.
    principal: Required. The principal whose access you want to check, in the
      form of the email address that represents that principal. For example,
      `alice@example.com` or `my-service-account@my-
      project.iam.gserviceaccount.com`. The principal must be a Google Account
      or a service account. Other types of principals are not supported.
  """

  fullResourceName = _messages.StringField(1)
  permission = _messages.StringField(2)
  principal = _messages.StringField(3)


class GoogleCloudPolicytroubleshooterV1BindingExplanation(_messages.Message):
  r"""Details about how a binding in a policy affects a principal's ability to
  use a permission.

  Enums:
    AccessValueValuesEnum: Required. Indicates whether _this binding_ provides
      the specified permission to the specified principal for the specified
      resource. This field does _not_ indicate whether the principal actually
      has the permission for the resource. There might be another binding that
      overrides this binding. To determine whether the principal actually has
      the permission, use the `access` field in the
      TroubleshootIamPolicyResponse.
    RelevanceValueValuesEnum: The relevance of this binding to the overall
      determination for the entire policy.
    RolePermissionValueValuesEnum: Indicates whether the role granted by this
      binding contains the specified permission.
    RolePermissionRelevanceValueValuesEnum: The relevance of the permission's
      existence, or nonexistence, in the role to the overall determination for
      the entire policy.

  Messages:
    MembershipsValue: Indicates whether each principal in the binding includes
      the principal specified in the request, either directly or indirectly.
      Each key identifies a principal in the binding, and each value indicates
      whether the principal in the binding includes the principal in the
      request. For example, suppose that a binding includes the following
      principals: * `user:alice@example.com` * `group:product-eng@example.com`
      You want to troubleshoot access for `user:bob@example.com`. This user is
      a principal of the group `group:product-eng@example.com`. For the first
      principal in the binding, the key is `user:alice@example.com`, and the
      `membership` field in the value is set to `MEMBERSHIP_NOT_INCLUDED`. For
      the second principal in the binding, the key is `group:product-
      eng@example.com`, and the `membership` field in the value is set to
      `MEMBERSHIP_INCLUDED`.

  Fields:
    access: Required. Indicates whether _this binding_ provides the specified
      permission to the specified principal for the specified resource. This
      field does _not_ indicate whether the principal actually has the
      permission for the resource. There might be another binding that
      overrides this binding. To determine whether the principal actually has
      the permission, use the `access` field in the
      TroubleshootIamPolicyResponse.
    condition: A condition expression that prevents this binding from granting
      access unless the expression evaluates to `true`. To learn about IAM
      Conditions, see https://cloud.google.com/iam/help/conditions/overview.
    memberships: Indicates whether each principal in the binding includes the
      principal specified in the request, either directly or indirectly. Each
      key identifies a principal in the binding, and each value indicates
      whether the principal in the binding includes the principal in the
      request. For example, suppose that a binding includes the following
      principals: * `user:alice@example.com` * `group:product-eng@example.com`
      You want to troubleshoot access for `user:bob@example.com`. This user is
      a principal of the group `group:product-eng@example.com`. For the first
      principal in the binding, the key is `user:alice@example.com`, and the
      `membership` field in the value is set to `MEMBERSHIP_NOT_INCLUDED`. For
      the second principal in the binding, the key is `group:product-
      eng@example.com`, and the `membership` field in the value is set to
      `MEMBERSHIP_INCLUDED`.
    relevance: The relevance of this binding to the overall determination for
      the entire policy.
    role: The role that this binding grants. For example,
      `roles/compute.serviceAgent`. For a complete list of predefined IAM
      roles, as well as the permissions in each role, see
      https://cloud.google.com/iam/help/roles/reference.
    rolePermission: Indicates whether the role granted by this binding
      contains the specified permission.
    rolePermissionRelevance: The relevance of the permission's existence, or
      nonexistence, in the role to the overall determination for the entire
      policy.
  """

  class AccessValueValuesEnum(_messages.Enum):
    r"""Required. Indicates whether _this binding_ provides the specified
    permission to the specified principal for the specified resource. This
    field does _not_ indicate whether the principal actually has the
    permission for the resource. There might be another binding that overrides
    this binding. To determine whether the principal actually has the
    permission, use the `access` field in the TroubleshootIamPolicyResponse.

    Values:
      ACCESS_STATE_UNSPECIFIED: Default value. This value is unused.
      GRANTED: The principal has the permission.
      NOT_GRANTED: The principal does not have the permission.
      UNKNOWN_CONDITIONAL: The principal has the permission only if a
        condition expression evaluates to `true`.
      UNKNOWN_INFO_DENIED: The sender of the request does not have access to
        all of the policies that Policy Troubleshooter needs to evaluate.
    """
    ACCESS_STATE_UNSPECIFIED = 0
    GRANTED = 1
    NOT_GRANTED = 2
    UNKNOWN_CONDITIONAL = 3
    UNKNOWN_INFO_DENIED = 4

  class RelevanceValueValuesEnum(_messages.Enum):
    r"""The relevance of this binding to the overall determination for the
    entire policy.

    Values:
      HEURISTIC_RELEVANCE_UNSPECIFIED: Default value. This value is unused.
      NORMAL: The data point has a limited effect on the result. Changing the
        data point is unlikely to affect the overall determination.
      HIGH: The data point has a strong effect on the result. Changing the
        data point is likely to affect the overall determination.
    """
    HEURISTIC_RELEVANCE_UNSPECIFIED = 0
    NORMAL = 1
    HIGH = 2

  class RolePermissionRelevanceValueValuesEnum(_messages.Enum):
    r"""The relevance of the permission's existence, or nonexistence, in the
    role to the overall determination for the entire policy.

    Values:
      HEURISTIC_RELEVANCE_UNSPECIFIED: Default value. This value is unused.
      NORMAL: The data point has a limited effect on the result. Changing the
        data point is unlikely to affect the overall determination.
      HIGH: The data point has a strong effect on the result. Changing the
        data point is likely to affect the overall determination.
    """
    HEURISTIC_RELEVANCE_UNSPECIFIED = 0
    NORMAL = 1
    HIGH = 2

  class RolePermissionValueValuesEnum(_messages.Enum):
    r"""Indicates whether the role granted by this binding contains the
    specified permission.

    Values:
      ROLE_PERMISSION_UNSPECIFIED: Default value. This value is unused.
      ROLE_PERMISSION_INCLUDED: The permission is included in the role.
      ROLE_PERMISSION_NOT_INCLUDED: The permission is not included in the
        role.
      ROLE_PERMISSION_UNKNOWN_INFO_DENIED: The sender of the request is not
        allowed to access the binding.
    """
    ROLE_PERMISSION_UNSPECIFIED = 0
    ROLE_PERMISSION_INCLUDED = 1
    ROLE_PERMISSION_NOT_INCLUDED = 2
    ROLE_PERMISSION_UNKNOWN_INFO_DENIED = 3

  @encoding.MapUnrecognizedFields('additionalProperties')
  class MembershipsValue(_messages.Message):
    r"""Indicates whether each principal in the binding includes the principal
    specified in the request, either directly or indirectly. Each key
    identifies a principal in the binding, and each value indicates whether
    the principal in the binding includes the principal in the request. For
    example, suppose that a binding includes the following principals: *
    `user:alice@example.com` * `group:product-eng@example.com` You want to
    troubleshoot access for `user:bob@example.com`. This user is a principal
    of the group `group:product-eng@example.com`. For the first principal in
    the binding, the key is `user:alice@example.com`, and the `membership`
    field in the value is set to `MEMBERSHIP_NOT_INCLUDED`. For the second
    principal in the binding, the key is `group:product-eng@example.com`, and
    the `membership` field in the value is set to `MEMBERSHIP_INCLUDED`.

    Messages:
      AdditionalProperty: An additional property for a MembershipsValue
        object.

    Fields:
      additionalProperties: Additional properties of type MembershipsValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a MembershipsValue object.

      Fields:
        key: Name of the additional property.
        value: A
          GoogleCloudPolicytroubleshooterV1BindingExplanationAnnotatedMembersh
          ip attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('GoogleCloudPolicytroubleshooterV1BindingExplanationAnnotatedMembership', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  access = _messages.EnumField('AccessValueValuesEnum', 1)
  condition = _messages.MessageField('GoogleTypeExpr', 2)
  memberships = _messages.MessageField('MembershipsValue', 3)
  relevance = _messages.EnumField('RelevanceValueValuesEnum', 4)
  role = _messages.StringField(5)
  rolePermission = _messages.EnumField('RolePermissionValueValuesEnum', 6)
  rolePermissionRelevance = _messages.EnumField('RolePermissionRelevanceValueValuesEnum', 7)


class GoogleCloudPolicytroubleshooterV1BindingExplanationAnnotatedMembership(_messages.Message):
  r"""Details about whether the binding includes the principal.

  Enums:
    MembershipValueValuesEnum: Indicates whether the binding includes the
      principal.
    RelevanceValueValuesEnum: The relevance of the principal's status to the
      overall determination for the binding.

  Fields:
    membership: Indicates whether the binding includes the principal.
    relevance: The relevance of the principal's status to the overall
      determination for the binding.
  """

  class MembershipValueValuesEnum(_messages.Enum):
    r"""Indicates whether the binding includes the principal.

    Values:
      MEMBERSHIP_UNSPECIFIED: Default value. This value is unused.
      MEMBERSHIP_INCLUDED: The binding includes the principal. The principal
        can be included directly or indirectly. For example: * A principal is
        included directly if that principal is listed in the binding. * A
        principal is included indirectly if that principal is in a Google
        group or Google Workspace domain that is listed in the binding.
      MEMBERSHIP_NOT_INCLUDED: The binding does not include the principal.
      MEMBERSHIP_UNKNOWN_INFO_DENIED: The sender of the request is not allowed
        to access the binding.
      MEMBERSHIP_UNKNOWN_UNSUPPORTED: The principal is an unsupported type.
        Only Google Accounts and service accounts are supported.
    """
    MEMBERSHIP_UNSPECIFIED = 0
    MEMBERSHIP_INCLUDED = 1
    MEMBERSHIP_NOT_INCLUDED = 2
    MEMBERSHIP_UNKNOWN_INFO_DENIED = 3
    MEMBERSHIP_UNKNOWN_UNSUPPORTED = 4

  class RelevanceValueValuesEnum(_messages.Enum):
    r"""The relevance of the principal's status to the overall determination
    for the binding.

    Values:
      HEURISTIC_RELEVANCE_UNSPECIFIED: Default value. This value is unused.
      NORMAL: The data point has a limited effect on the result. Changing the
        data point is unlikely to affect the overall determination.
      HIGH: The data point has a strong effect on the result. Changing the
        data point is likely to affect the overall determination.
    """
    HEURISTIC_RELEVANCE_UNSPECIFIED = 0
    NORMAL = 1
    HIGH = 2

  membership = _messages.EnumField('MembershipValueValuesEnum', 1)
  relevance = _messages.EnumField('RelevanceValueValuesEnum', 2)


class GoogleCloudPolicytroubleshooterV1ExplainedPolicy(_messages.Message):
  r"""Details about how a specific IAM Policy contributed to the access check.

  Enums:
    AccessValueValuesEnum: Indicates whether _this policy_ provides the
      specified permission to the specified principal for the specified
      resource. This field does _not_ indicate whether the principal actually
      has the permission for the resource. There might be another policy that
      overrides this policy. To determine whether the principal actually has
      the permission, use the `access` field in the
      TroubleshootIamPolicyResponse.
    RelevanceValueValuesEnum: The relevance of this policy to the overall
      determination in the TroubleshootIamPolicyResponse. If the sender of the
      request does not have access to the policy, this field is omitted.

  Fields:
    access: Indicates whether _this policy_ provides the specified permission
      to the specified principal for the specified resource. This field does
      _not_ indicate whether the principal actually has the permission for the
      resource. There might be another policy that overrides this policy. To
      determine whether the principal actually has the permission, use the
      `access` field in the TroubleshootIamPolicyResponse.
    bindingExplanations: Details about how each binding in the policy affects
      the principal's ability, or inability, to use the permission for the
      resource. If the sender of the request does not have access to the
      policy, this field is omitted.
    fullResourceName: The full resource name that identifies the resource. For
      example, `//compute.googleapis.com/projects/my-project/zones/us-
      central1-a/instances/my-instance`. If the sender of the request does not
      have access to the policy, this field is omitted. For examples of full
      resource names for Google Cloud services, see
      https://cloud.google.com/iam/help/troubleshooter/full-resource-names.
    policy: The IAM policy attached to the resource. If the sender of the
      request does not have access to the policy, this field is empty.
    relevance: The relevance of this policy to the overall determination in
      the TroubleshootIamPolicyResponse. If the sender of the request does not
      have access to the policy, this field is omitted.
  """

  class AccessValueValuesEnum(_messages.Enum):
    r"""Indicates whether _this policy_ provides the specified permission to
    the specified principal for the specified resource. This field does _not_
    indicate whether the principal actually has the permission for the
    resource. There might be another policy that overrides this policy. To
    determine whether the principal actually has the permission, use the
    `access` field in the TroubleshootIamPolicyResponse.

    Values:
      ACCESS_STATE_UNSPECIFIED: Default value. This value is unused.
      GRANTED: The principal has the permission.
      NOT_GRANTED: The principal does not have the permission.
      UNKNOWN_CONDITIONAL: The principal has the permission only if a
        condition expression evaluates to `true`.
      UNKNOWN_INFO_DENIED: The sender of the request does not have access to
        all of the policies that Policy Troubleshooter needs to evaluate.
    """
    ACCESS_STATE_UNSPECIFIED = 0
    GRANTED = 1
    NOT_GRANTED = 2
    UNKNOWN_CONDITIONAL = 3
    UNKNOWN_INFO_DENIED = 4

  class RelevanceValueValuesEnum(_messages.Enum):
    r"""The relevance of this policy to the overall determination in the
    TroubleshootIamPolicyResponse. If the sender of the request does not have
    access to the policy, this field is omitted.

    Values:
      HEURISTIC_RELEVANCE_UNSPECIFIED: Default value. This value is unused.
      NORMAL: The data point has a limited effect on the result. Changing the
        data point is unlikely to affect the overall determination.
      HIGH: The data point has a strong effect on the result. Changing the
        data point is likely to affect the overall determination.
    """
    HEURISTIC_RELEVANCE_UNSPECIFIED = 0
    NORMAL = 1
    HIGH = 2

  access = _messages.EnumField('AccessValueValuesEnum', 1)
  bindingExplanations = _messages.MessageField('GoogleCloudPolicytroubleshooterV1BindingExplanation', 2, repeated=True)
  fullResourceName = _messages.StringField(3)
  policy = _messages.MessageField('GoogleIamV1Policy', 4)
  relevance = _messages.EnumField('RelevanceValueValuesEnum', 5)


class GoogleCloudPolicytroubleshooterV1TroubleshootIamPolicyRequest(_messages.Message):
  r"""Request for TroubleshootIamPolicy.

  Fields:
    accessTuple: The information to use for checking whether a principal has a
      permission for a resource.
  """

  accessTuple = _messages.MessageField('GoogleCloudPolicytroubleshooterV1AccessTuple', 1)


class GoogleCloudPolicytroubleshooterV1TroubleshootIamPolicyResponse(_messages.Message):
  r"""Response for TroubleshootIamPolicy.

  Enums:
    AccessValueValuesEnum: Indicates whether the principal has the specified
      permission for the specified resource, based on evaluating all of the
      applicable IAM policies.

  Fields:
    access: Indicates whether the principal has the specified permission for
      the specified resource, based on evaluating all of the applicable IAM
      policies.
    errors: The general errors contained in the troubleshooting response.
    explainedPolicies: List of IAM policies that were evaluated to check the
      principal's permissions, with annotations to indicate how each policy
      contributed to the final result. The list of policies can include the
      policy for the resource itself. It can also include policies that are
      inherited from higher levels of the resource hierarchy, including the
      organization, the folder, and the project. To learn more about the
      resource hierarchy, see https://cloud.google.com/iam/help/resource-
      hierarchy.
  """

  class AccessValueValuesEnum(_messages.Enum):
    r"""Indicates whether the principal has the specified permission for the
    specified resource, based on evaluating all of the applicable IAM
    policies.

    Values:
      ACCESS_STATE_UNSPECIFIED: Default value. This value is unused.
      GRANTED: The principal has the permission.
      NOT_GRANTED: The principal does not have the permission.
      UNKNOWN_CONDITIONAL: The principal has the permission only if a
        condition expression evaluates to `true`.
      UNKNOWN_INFO_DENIED: The sender of the request does not have access to
        all of the policies that Policy Troubleshooter needs to evaluate.
    """
    ACCESS_STATE_UNSPECIFIED = 0
    GRANTED = 1
    NOT_GRANTED = 2
    UNKNOWN_CONDITIONAL = 3
    UNKNOWN_INFO_DENIED = 4

  access = _messages.EnumField('AccessValueValuesEnum', 1)
  errors = _messages.MessageField('GoogleRpcStatus', 2, repeated=True)
  explainedPolicies = _messages.MessageField('GoogleCloudPolicytroubleshooterV1ExplainedPolicy', 3, repeated=True)


class GoogleIamV1AuditConfig(_messages.Message):
  r"""Specifies the audit configuration for a service. The configuration
  determines which permission types are logged, and what identities, if any,
  are exempted from logging. An AuditConfig must have one or more
  AuditLogConfigs. If there are AuditConfigs for both `allServices` and a
  specific service, the union of the two AuditConfigs is used for that
  service: the log_types specified in each AuditConfig are enabled, and the
  exempted_members in each AuditLogConfig are exempted. Example Policy with
  multiple AuditConfigs: { "audit_configs": [ { "service": "allServices",
  "audit_log_configs": [ { "log_type": "DATA_READ", "exempted_members": [
  "user:jose@example.com" ] }, { "log_type": "DATA_WRITE" }, { "log_type":
  "ADMIN_READ" } ] }, { "service": "sampleservice.googleapis.com",
  "audit_log_configs": [ { "log_type": "DATA_READ" }, { "log_type":
  "DATA_WRITE", "exempted_members": [ "user:aliya@example.com" ] } ] } ] } For
  sampleservice, this policy enables DATA_READ, DATA_WRITE and ADMIN_READ
  logging. It also exempts `jose@example.com` from DATA_READ logging, and
  `aliya@example.com` from DATA_WRITE logging.

  Fields:
    auditLogConfigs: The configuration for logging of each type of permission.
    service: Specifies a service that will be enabled for audit logging. For
      example, `storage.googleapis.com`, `cloudsql.googleapis.com`.
      `allServices` is a special value that covers all services.
  """

  auditLogConfigs = _messages.MessageField('GoogleIamV1AuditLogConfig', 1, repeated=True)
  service = _messages.StringField(2)


class GoogleIamV1AuditLogConfig(_messages.Message):
  r"""Provides the configuration for logging a type of permissions. Example: {
  "audit_log_configs": [ { "log_type": "DATA_READ", "exempted_members": [
  "user:jose@example.com" ] }, { "log_type": "DATA_WRITE" } ] } This enables
  'DATA_READ' and 'DATA_WRITE' logging, while exempting jose@example.com from
  DATA_READ logging.

  Enums:
    LogTypeValueValuesEnum: The log type that this config enables.

  Fields:
    exemptedMembers: Specifies the identities that do not cause logging for
      this type of permission. Follows the same format of Binding.members.
    logType: The log type that this config enables.
  """

  class LogTypeValueValuesEnum(_messages.Enum):
    r"""The log type that this config enables.

    Values:
      LOG_TYPE_UNSPECIFIED: Default case. Should never be this.
      ADMIN_READ: Admin reads. Example: CloudIAM getIamPolicy
      DATA_WRITE: Data writes. Example: CloudSQL Users create
      DATA_READ: Data reads. Example: CloudSQL Users list
    """
    LOG_TYPE_UNSPECIFIED = 0
    ADMIN_READ = 1
    DATA_WRITE = 2
    DATA_READ = 3

  exemptedMembers = _messages.StringField(1, repeated=True)
  logType = _messages.EnumField('LogTypeValueValuesEnum', 2)


class GoogleIamV1Binding(_messages.Message):
  r"""Associates `members`, or principals, with a `role`.

  Fields:
    condition: The condition that is associated with this binding. If the
      condition evaluates to `true`, then this binding applies to the current
      request. If the condition evaluates to `false`, then this binding does
      not apply to the current request. However, a different role binding
      might grant the same role to one or more of the principals in this
      binding. To learn which resources support conditions in their IAM
      policies, see the [IAM
      documentation](https://cloud.google.com/iam/help/conditions/resource-
      policies).
    members: Specifies the principals requesting access for a Google Cloud
      resource. `members` can have the following values: * `allUsers`: A
      special identifier that represents anyone who is on the internet; with
      or without a Google account. * `allAuthenticatedUsers`: A special
      identifier that represents anyone who is authenticated with a Google
      account or a service account. Does not include identities that come from
      external identity providers (IdPs) through identity federation. *
      `user:{emailid}`: An email address that represents a specific Google
      account. For example, `alice@example.com` . *
      `serviceAccount:{emailid}`: An email address that represents a Google
      service account. For example, `my-other-
      app@appspot.gserviceaccount.com`. *
      `serviceAccount:{projectid}.svc.id.goog[{namespace}/{kubernetes-sa}]`:
      An identifier for a [Kubernetes service
      account](https://cloud.google.com/kubernetes-engine/docs/how-
      to/kubernetes-service-accounts). For example, `my-
      project.svc.id.goog[my-namespace/my-kubernetes-sa]`. *
      `group:{emailid}`: An email address that represents a Google group. For
      example, `admins@example.com`. * `domain:{domain}`: The G Suite domain
      (primary) that represents all the users of that domain. For example,
      `google.com` or `example.com`. * `principal://iam.googleapis.com/locatio
      ns/global/workforcePools/{pool_id}/subject/{subject_attribute_value}`: A
      single identity in a workforce identity pool. * `principalSet://iam.goog
      leapis.com/locations/global/workforcePools/{pool_id}/group/{group_id}`:
      All workforce identities in a group. * `principalSet://iam.googleapis.co
      m/locations/global/workforcePools/{pool_id}/attribute.{attribute_name}/{
      attribute_value}`: All workforce identities with a specific attribute
      value. * `principalSet://iam.googleapis.com/locations/global/workforcePo
      ols/{pool_id}/*`: All identities in a workforce identity pool. * `princi
      pal://iam.googleapis.com/projects/{project_number}/locations/global/work
      loadIdentityPools/{pool_id}/subject/{subject_attribute_value}`: A single
      identity in a workload identity pool. * `principalSet://iam.googleapis.c
      om/projects/{project_number}/locations/global/workloadIdentityPools/{poo
      l_id}/group/{group_id}`: A workload identity pool group. * `principalSet
      ://iam.googleapis.com/projects/{project_number}/locations/global/workloa
      dIdentityPools/{pool_id}/attribute.{attribute_name}/{attribute_value}`:
      All identities in a workload identity pool with a certain attribute. * `
      principalSet://iam.googleapis.com/projects/{project_number}/locations/gl
      obal/workloadIdentityPools/{pool_id}/*`: All identities in a workload
      identity pool. * `deleted:user:{emailid}?uid={uniqueid}`: An email
      address (plus unique identifier) representing a user that has been
      recently deleted. For example,
      `alice@example.com?uid=123456789012345678901`. If the user is recovered,
      this value reverts to `user:{emailid}` and the recovered user retains
      the role in the binding. *
      `deleted:serviceAccount:{emailid}?uid={uniqueid}`: An email address
      (plus unique identifier) representing a service account that has been
      recently deleted. For example, `my-other-
      app@appspot.gserviceaccount.com?uid=123456789012345678901`. If the
      service account is undeleted, this value reverts to
      `serviceAccount:{emailid}` and the undeleted service account retains the
      role in the binding. * `deleted:group:{emailid}?uid={uniqueid}`: An
      email address (plus unique identifier) representing a Google group that
      has been recently deleted. For example,
      `admins@example.com?uid=123456789012345678901`. If the group is
      recovered, this value reverts to `group:{emailid}` and the recovered
      group retains the role in the binding. * `deleted:principal://iam.google
      apis.com/locations/global/workforcePools/{pool_id}/subject/{subject_attr
      ibute_value}`: Deleted single identity in a workforce identity pool. For
      example, `deleted:principal://iam.googleapis.com/locations/global/workfo
      rcePools/my-pool-id/subject/my-subject-attribute-value`.
    role: Role that is assigned to the list of `members`, or principals. For
      example, `roles/viewer`, `roles/editor`, or `roles/owner`. For an
      overview of the IAM roles and permissions, see the [IAM
      documentation](https://cloud.google.com/iam/docs/roles-overview). For a
      list of the available pre-defined roles, see
      [here](https://cloud.google.com/iam/docs/understanding-roles).
  """

  condition = _messages.MessageField('GoogleTypeExpr', 1)
  members = _messages.StringField(2, repeated=True)
  role = _messages.StringField(3)


class GoogleIamV1Policy(_messages.Message):
  r"""An Identity and Access Management (IAM) policy, which specifies access
  controls for Google Cloud resources. A `Policy` is a collection of
  `bindings`. A `binding` binds one or more `members`, or principals, to a
  single `role`. Principals can be user accounts, service accounts, Google
  groups, and domains (such as G Suite). A `role` is a named list of
  permissions; each `role` can be an IAM predefined role or a user-created
  custom role. For some types of Google Cloud resources, a `binding` can also
  specify a `condition`, which is a logical expression that allows access to a
  resource only if the expression evaluates to `true`. A condition can add
  constraints based on attributes of the request, the resource, or both. To
  learn which resources support conditions in their IAM policies, see the [IAM
  documentation](https://cloud.google.com/iam/help/conditions/resource-
  policies). **JSON example:** ``` { "bindings": [ { "role":
  "roles/resourcemanager.organizationAdmin", "members": [
  "user:mike@example.com", "group:admins@example.com", "domain:google.com",
  "serviceAccount:my-project-id@appspot.gserviceaccount.com" ] }, { "role":
  "roles/resourcemanager.organizationViewer", "members": [
  "user:eve@example.com" ], "condition": { "title": "expirable access",
  "description": "Does not grant access after Sep 2020", "expression":
  "request.time < timestamp('2020-10-01T00:00:00.000Z')", } } ], "etag":
  "BwWWja0YfJA=", "version": 3 } ``` **YAML example:** ``` bindings: -
  members: - user:mike@example.com - group:admins@example.com -
  domain:google.com - serviceAccount:my-project-id@appspot.gserviceaccount.com
  role: roles/resourcemanager.organizationAdmin - members: -
  user:eve@example.com role: roles/resourcemanager.organizationViewer
  condition: title: expirable access description: Does not grant access after
  Sep 2020 expression: request.time < timestamp('2020-10-01T00:00:00.000Z')
  etag: BwWWja0YfJA= version: 3 ``` For a description of IAM and its features,
  see the [IAM documentation](https://cloud.google.com/iam/docs/).

  Fields:
    auditConfigs: Specifies cloud audit logging configuration for this policy.
    bindings: Associates a list of `members`, or principals, with a `role`.
      Optionally, may specify a `condition` that determines how and when the
      `bindings` are applied. Each of the `bindings` must contain at least one
      principal. The `bindings` in a `Policy` can refer to up to 1,500
      principals; up to 250 of these principals can be Google groups. Each
      occurrence of a principal counts towards these limits. For example, if
      the `bindings` grant 50 different roles to `user:alice@example.com`, and
      not to any other principal, then you can add another 1,450 principals to
      the `bindings` in the `Policy`.
    etag: `etag` is used for optimistic concurrency control as a way to help
      prevent simultaneous updates of a policy from overwriting each other. It
      is strongly suggested that systems make use of the `etag` in the read-
      modify-write cycle to perform policy updates in order to avoid race
      conditions: An `etag` is returned in the response to `getIamPolicy`, and
      systems are expected to put that etag in the request to `setIamPolicy`
      to ensure that their change will be applied to the same version of the
      policy. **Important:** If you use IAM Conditions, you must include the
      `etag` field whenever you call `setIamPolicy`. If you omit this field,
      then IAM allows you to overwrite a version `3` policy with a version `1`
      policy, and all of the conditions in the version `3` policy are lost.
    version: Specifies the format of the policy. Valid values are `0`, `1`,
      and `3`. Requests that specify an invalid value are rejected. Any
      operation that affects conditional role bindings must specify version
      `3`. This requirement applies to the following operations: * Getting a
      policy that includes a conditional role binding * Adding a conditional
      role binding to a policy * Changing a conditional role binding in a
      policy * Removing any role binding, with or without a condition, from a
      policy that includes conditions **Important:** If you use IAM
      Conditions, you must include the `etag` field whenever you call
      `setIamPolicy`. If you omit this field, then IAM allows you to overwrite
      a version `3` policy with a version `1` policy, and all of the
      conditions in the version `3` policy are lost. If a policy does not
      include any conditions, operations on that policy may specify any valid
      version or leave the field unset. To learn which resources support
      conditions in their IAM policies, see the [IAM
      documentation](https://cloud.google.com/iam/help/conditions/resource-
      policies).
  """

  auditConfigs = _messages.MessageField('GoogleIamV1AuditConfig', 1, repeated=True)
  bindings = _messages.MessageField('GoogleIamV1Binding', 2, repeated=True)
  etag = _messages.BytesField(3)
  version = _messages.IntegerField(4, variant=_messages.Variant.INT32)


class GoogleRpcStatus(_messages.Message):
  r"""The `Status` type defines a logical error model that is suitable for
  different programming environments, including REST APIs and RPC APIs. It is
  used by [gRPC](https://github.com/grpc). Each `Status` message contains
  three pieces of data: error code, error message, and error details. You can
  find out more about this error model and how to work with it in the [API
  Design Guide](https://cloud.google.com/apis/design/errors).

  Messages:
    DetailsValueListEntry: A DetailsValueListEntry object.

  Fields:
    code: The status code, which should be an enum value of google.rpc.Code.
    details: A list of messages that carry the error details. There is a
      common set of message types for APIs to use.
    message: A developer-facing error message, which should be in English. Any
      user-facing error message should be localized and sent in the
      google.rpc.Status.details field, or localized by the client.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class DetailsValueListEntry(_messages.Message):
    r"""A DetailsValueListEntry object.

    Messages:
      AdditionalProperty: An additional property for a DetailsValueListEntry
        object.

    Fields:
      additionalProperties: Properties of the object. Contains field @type
        with type URL.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a DetailsValueListEntry object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  code = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  details = _messages.MessageField('DetailsValueListEntry', 2, repeated=True)
  message = _messages.StringField(3)


class GoogleTypeExpr(_messages.Message):
  r"""Represents a textual expression in the Common Expression Language (CEL)
  syntax. CEL is a C-like expression language. The syntax and semantics of CEL
  are documented at https://github.com/google/cel-spec. Example (Comparison):
  title: "Summary size limit" description: "Determines if a summary is less
  than 100 chars" expression: "document.summary.size() < 100" Example
  (Equality): title: "Requestor is owner" description: "Determines if
  requestor is the document owner" expression: "document.owner ==
  request.auth.claims.email" Example (Logic): title: "Public documents"
  description: "Determine whether the document should be publicly visible"
  expression: "document.type != 'private' && document.type != 'internal'"
  Example (Data Manipulation): title: "Notification string" description:
  "Create a notification string with a timestamp." expression: "'New message
  received at ' + string(document.create_time)" The exact variables and
  functions that may be referenced within an expression are determined by the
  service that evaluates it. See the service documentation for additional
  information.

  Fields:
    description: Optional. Description of the expression. This is a longer
      text which describes the expression, e.g. when hovered over it in a UI.
    expression: Textual representation of an expression in Common Expression
      Language syntax.
    location: Optional. String indicating the location of the expression for
      error reporting, e.g. a file name and a position in the file.
    title: Optional. Title for the expression, i.e. a short string describing
      its purpose. This can be used e.g. in UIs which allow to enter the
      expression.
  """

  description = _messages.StringField(1)
  expression = _messages.StringField(2)
  location = _messages.StringField(3)
  title = _messages.StringField(4)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Copy an Artifact Registry repository."""


from googlecloudsdk.api_lib.util import waiter
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.artifacts import flags
from googlecloudsdk.command_lib.artifacts import requests
from googlecloudsdk.core import log
from googlecloudsdk.core import resources
from googlecloudsdk.core.console import progress_tracker
from googlecloudsdk.core.util import retry


@base.UniverseCompatible
@base.ReleaseTracks(base.ReleaseTrack.GA)
@base.Hidden
class Copy(base.Command):
  """Copy an Artifact Registry repository."""

  detailed_help = {
      'BRIEF': 'Copy an Artifact Registry repository.',
      'DESCRIPTION': """
          Copy all artifacts within an Artifact Registry repository to another repository.

          Note that this is pull-based, so default arguments will apply to the destination repository.
      """,
      'EXAMPLES': """\
          To copy artifacts from `repo1` in project `proj1` and location `us` to `repo2` in project `proj2` and location `asia`:

            $ {command} repo2 --project=proj2 --location=asia --source-repo=projects/proj1/locations/us/repositories/repo1
      """,
  }

  @staticmethod
  def Args(parser):
    """Set up arguments for this command.

    Args:
      parser: An argparse.ArgumentParser.
    """
    flags.GetRepoArg().AddToParser(parser)
    base.ASYNC_FLAG.AddToParser(parser)
    parser.add_argument(
        '--source-repo',
        metavar='SOURCE_REPOSITORY',
        required=True,
        help='The source repository path to copy artifacts from.',
    )
    parser.add_argument(
        '--continue-on-skipped-version',
        action='store_true',
        default=False,
        help=(
            'If true, the repo copy operation will continue when a'
            ' non-transient version copy error occurs, rather than failing the'
            ' entire copy operation. The skipped version name and error will be'
            ' stored in the lro metadata and response.'
        ),
    )
    parser.add_argument(
        '--max-version-age-days',
        type=int,
        default=0,
        help=(
            'If set, only versions updated within the specified number of days'
            " (24-hour periods) up to the operation's start time will be"
            ' copied. If unset or set to 0, all versions will be copied. This'
            ' arg provides a convenient way to speed up regular repository'
            ' copies by skipping older versions.'
        ),
    )
    parser.add_argument(
        '--include-all-attachments',
        action='store_true',
        default=False,
        help='Include all attachments for each artifact version copied.',
    )
    parser.add_argument(
        '--exclude-all-tags',
        action='store_true',
        default=False,
        help='Exclude all tags for each artifact version copied.',
    )

  def Run(self, args):
    """Run the repository copy command."""
    # Call CopyRepository API.
    client = requests.GetClient()
    repo_ref = args.CONCEPTS.repository.Parse()
    op = requests.CopyRepository(
        args.source_repo,
        repo_ref.RelativeName(),
        args.continue_on_skipped_version,
        args.max_version_age_days,
        args.include_all_attachments,
        args.exclude_all_tags,
    )
    op_ref = resources.REGISTRY.ParseRelativeName(
        op.name, collection='artifactregistry.projects.locations.operations'
    )

    # Log operation details and return if not waiting.
    log.status.Print(
        'Copy request issued from [{}] to [{}].\nCreated operation [{}].'
        .format(
            args.source_repo, repo_ref.RelativeName(), op_ref.RelativeName()
        )
    )
    if args.async_:  # Return early if --async is specified.
      return None

    # Progress tracker for polling loop.
    spinner_message = 'Copying artifacts'
    progress_info = {'copied': 0, 'total': 0, 'skipped': 0}
    with progress_tracker.ProgressTracker(
        spinner_message,
        detail_message_callback=lambda: self._DetailMessage(progress_info),
        autotick=True,
    ):
      # Before polling, let user know to press CTRL-C to stop.
      log.status.Print('Press CTRL-C to stop waiting on the operation.')

      # Poll operation indefinitely.
      poller = waiter.CloudOperationPollerNoResources(
          client.projects_locations_operations
      )
      retryer = retry.Retryer(max_wait_ms=None)
      try:
        operation = retryer.RetryOnResult(
            lambda: self._PollOperation(poller, op_ref, progress_info),
            should_retry_if=lambda op, state: op and not op.done,
            sleep_ms=2000,
        )
        if operation.error:
          log.status.Print(
              '\nOperation [{}] failed: {}'.format(
                  op_ref.RelativeName(), operation.error.message
              )
          )
        return None
      except retry.WaitException:
        # This block should not be reached with max_wait_ms=None
        log.error('Error: Copy operation wait unexpectedly timed out.')
        return None
      except Exception as e:  # pylint: disable=broad-exception-caught
        log.fatal('An unexpected error occurred: {}'.format(e))
        return None

  def _DetailMessage(self, progress_info):
    """Callback to update the progress tracker message."""
    copied = progress_info['copied']
    total = progress_info['total']
    skipped = progress_info['skipped']

    if total == 0:
      return ' operation metadata not yet available'

    progress = (copied + skipped) / total * 100
    msg = ' {:.1f}% complete ({} of {} versions copied'.format(
        progress, copied, total
    )
    if skipped > 0:
      msg += ', {} skipped'.format(skipped)
    msg += ')'
    return msg

  def _PollOperation(self, poller, op_ref, progress_info):
    """Polls the operation and updates progress_info from metadata."""
    try:
      operation = poller.Poll(op_ref)
    except waiter.PollException as e:
      # Handle potential polling errors if necessary
      log.debug('Polling error: %s', e)
      return None  # Continue polling

    if (
        operation
        and operation.metadata
        and operation.metadata.additionalProperties
    ):
      props = {p.key: p.value for p in operation.metadata.additionalProperties}
      if 'versionsCopiedCount' in props:
        progress_info['copied'] = props['versionsCopiedCount'].integer_value
      if 'totalVersionsCount' in props:
        progress_info['total'] = props['totalVersionsCount'].integer_value
      if 'skippedVersionErrorCount' in props:
        progress_info['skipped'] = props[
            'skippedVersionErrorCount'
        ].integer_value

    return operation



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Code that's shared between multiple org firewall policies subcommands."""


import re

from googlecloudsdk.command_lib.resource_manager import tag_utils

# Regex explained:
# [0-9]+/[a-zA-Z0-9][^\"\\\\'/]* - namespaced format for tag key
# [0-9]+ - org id or project number
# [a-zA-Z0-9] - starts with an alphanumeric character
# [^\"\\\\'/]* - followed by any number of characters except single quote,
#                double quote, forward slash and backslash
NAMESPACED_TAG_KEY_PATTERN = r"[0-9]+/[a-zA-Z0-9][^\"\\\\'/]*"
NAMESPACED_TAG_KEY_REGEX = re.compile(NAMESPACED_TAG_KEY_PATTERN)

SHORT_NAME_VALUE_PATTERN = r"[a-zA-Z0-9][^\"\\'/]*"

NAMESPACED_TAG_VALUE_PATTERN = (
    f"({NAMESPACED_TAG_KEY_PATTERN})" + "/" + f"({SHORT_NAME_VALUE_PATTERN})"
)
NAMESPACED_TAG_VALUE_REGEX = re.compile(NAMESPACED_TAG_VALUE_PATTERN)
NUMERIC_TAG_VALUE_REGEX = re.compile(r"tagValues/[0-9]+")


def GetResourceManagerTags(resource_manager_tags):
  """Returns a map of resource manager tags, translating namespaced tags if needed.

  Args:
    resource_manager_tags: Map of resource manager tag key value pairs with
      either namespaced or numeric format.

  Returns:
    Map of resource manager tag key value pairs with either namespaced or
    numeric format.
  """

  ret_resource_manager_tags = {}
  for key, value in resource_manager_tags.items():
    # If the tag value is in namespaced format, extract the namespaced
    # key and short name value from it to pass to the API.
    namespaced_tag_value_match = NAMESPACED_TAG_VALUE_REGEX.fullmatch(value)
    if namespaced_tag_value_match:
      key = namespaced_tag_value_match.group(1)
      value = namespaced_tag_value_match.group(2)
    elif NAMESPACED_TAG_KEY_REGEX.fullmatch(
        key
    ) and NUMERIC_TAG_VALUE_REGEX.fullmatch(value):
      # If the tag key is in namespaced format, but the value is numeric,
      # translate the key to numeric format to pass to the API.
      key = tag_utils.GetNamespacedResource(key, tag_utils.TAG_KEYS).name
    ret_resource_manager_tags[key] = value

  return ret_resource_manager_tags



"""Generated message classes for batch version v1alpha1.

An API to manage the running of Batch resources on Google Cloud Platform.
"""
# NOTE: This file is autogenerated and should not be edited by hand.

from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding


package = 'batch'


class OperationMetadata(_messages.Message):
  r"""Represents the metadata of the long-running operation.

  Fields:
    apiVersion: Output only. API version used to start the operation.
    createTime: Output only. The time the operation was created.
    endTime: Output only. The time the operation finished running.
    requestedCancellation: Output only. Identifies whether the user has
      requested cancellation of the operation. Operations that have
      successfully been cancelled have Operation.error value with a
      google.rpc.Status.code of 1, corresponding to `Code.CANCELLED`.
    statusMessage: Output only. Human-readable status of the operation, if
      any.
    target: Output only. Server-defined resource path for the target of the
      operation.
    verb: Output only. Name of the verb executed by the operation.
  """

  apiVersion = _messages.StringField(1)
  createTime = _messages.StringField(2)
  endTime = _messages.StringField(3)
  requestedCancellation = _messages.BooleanField(4)
  statusMessage = _messages.StringField(5)
  target = _messages.StringField(6)
  verb = _messages.StringField(7)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')
